@@ -27,6 +27,10 @@ def get_store(request):
 def stripped_subject(mlist, subject):
     if mlist is None:
         return subject
+    if not subject:
+        return u"(no subject)"
+    if not mlist.subject_prefix:
+        return subject
     if subject.lower().startswith(mlist.subject_prefix.lower()):
         subject = subject[len(mlist.subject_prefix) : ]
     return subject
