@@ -22,10 +22,10 @@
 import datetime
 import re
 
+import django.utils.simplejson as json
 from dateutil.tz import tzoffset
 from django import template
 from django.utils.datastructures import SortedDict
-from django.templatetags.tz import localtime
 from django.utils.html import conditional_escape
 from django.utils.safestring import mark_safe
 from django.utils.timezone import utc
@@ -67,7 +67,6 @@ def to_date(month, year):
 
 @register.filter
 def strip_page(value):
-    print repr(value), repr(value)[-2]
     if not value:
         return value
     if value.endswith('/') and value[-3] == '/':
@@ -205,3 +204,13 @@ def add_to_query_string(context, *args, **kwargs):
     for key, value in new_qs_elements.iteritems():
         qs[key] = value
     return qs.urlencode()
+
+
+@register.filter
+def until(value, limit):
+    return value.partition(limit)[0]
+
+
+@register.filter
+def to_json(value):
+    return json.dumps(value)
