@@ -21,9 +21,12 @@
 
 import datetime
 
-from hyperkitty.tests.utils import TestCase
+from django.http import HttpRequest
 
-from hyperkitty.lib.view_helpers import get_display_dates, paginate
+from hyperkitty.lib.view_helpers import get_display_dates, show_mlist
+from hyperkitty.lib.paginator import paginate
+
+from hyperkitty.tests.utils import TestCase
 
 
 class GetDisplayDatesTestCase(TestCase):
@@ -95,3 +98,44 @@ class PaginateTestCase(TestCase):
                          [1, '...', 96, 97, 98, 99, 100])
         self.assertEqual(paginate(objects, 100).page_range,
                          [1, '...', 97, 98, 99, 100])
+
+
+#
+# view_helpers.show_mlist()
+#
+
+class FakeKSList(object):
+    def __init__(self, name):
+        self.name = name
+
+class ShowMlistTestCase(TestCase):
+
+    def _do_test(self, listdomain, vhost, expected):
+        mlist = FakeKSList("test@%s" % listdomain)
+        req = HttpRequest()
+        req.META["HTTP_HOST"] = vhost
+        self.assertEqual(show_mlist(mlist, req), expected)
+
+    def test_same_domain(self):
+        self._do_test("example.com", "example.com", True)
+        self._do_test("lists.example.com", "lists.example.com", True)
+
+    def test_web_subdomain(self):
+        self._do_test("example.com", "www.example.com", True)
+        self._do_test("example.com", "lists.example.com", True)
+
+    def test_mail_subdomain(self):
+        self._do_test("lists.example.com", "example.com", True)
+
+    def test_different_subdomains(self):
+        self._do_test("lists.example.com", "archives.example.com", True)
+
+    def test_different_domains(self):
+        self._do_test("example.com", "another-example.com", False)
+        self._do_test("lists.example.com", "archives.another-example.com", False)
+
+    def test_single_component_domain(self):
+        self._do_test("intranet", "intranet", True)
+
+    def test_different_single_component_domain(self):
+        self._do_test("intranet", "extranet", False)
