@@ -19,7 +19,7 @@
 # Author: Aurelien Bompard <abompard@fedoraproject.org>
 #
 
-from django.test import TestCase
+from hyperkitty.tests.utils import TestCase
 
 from hyperkitty.templatetags.hk_generic import snip_quoted
 
