@@ -21,74 +21,82 @@
 #
 
 import datetime
+from tempfile import mkdtemp
+from shutil import rmtree
 
 from mock import Mock
 
 import django.utils.simplejson as json
 from hyperkitty.tests.utils import TestCase
-from django.test.client import Client, RequestFactory
-from django.contrib.auth.models import User, AnonymousUser
+from django.contrib.auth.models import User
 from django.core.urlresolvers import reverse
 from mailman.email.message import Message
+from mailman.interfaces.archiver import ArchivePolicy
 
 import kittystore
 from kittystore.utils import get_message_id_hash
-from kittystore.test import FakeList
+from kittystore.test import FakeList, SettingsModule
+
+from hyperkitty.models import LastView
+
+
+
+def genericAssertRedirects(self, response, target):
+    # A version of assertRedirects that works with view responses
+    if hasattr(response, "client"):
+        return TestCase.assertRedirects(self, response, target)
+    self.assertEqual(response.status_code, 302)
+    self.assertEqual(response._headers["location"][1], target)
 
-from hyperkitty.models import Rating, LastView
 
 
 class AccountViewsTestCase(TestCase):
 
     def setUp(self):
-        self.client = Client()
+        store = kittystore.get_store(SettingsModule(),
+                                     debug=False, auto_create=True)
+        self.client.defaults = {"kittystore.store": store,
+                                "HTTP_USER_AGENT": "testbot",
+                                }
+        self.user = User.objects.create_user('testuser', 'test@example.com', 'testPass')
 
     def test_login(self):
         # Try to access user profile (private data) without logging in
-        response = self.client.get(reverse('user_profile'))
-        self.assertRedirects(response, "%s?next=%s" % (reverse('user_login'), reverse('user_profile')))
+        response = self.client.get(reverse("user_profile"))
+        self.assertRedirects(response,
+                "%s?next=%s" % (reverse('user_login'), reverse("user_profile")))
 
     def test_profile(self):
-        User.objects.create_user('testuser', 'test@example.com', 'testPass')
         self.client.login(username='testuser', password='testPass')
-
-        response = self.client.get(reverse('user_profile'))
+        response = self.client.get(reverse("user_profile"))
         self.assertEqual(response.status_code, 200)
 
-        # Verify that user_profile is present in request context
-        self.assertTrue('user_profile' in response.context)
-
-        # Verify karma for newly created user is 1
-        self.assertEqual(response.context['user_profile'].karma, 1)
-
-
     def test_registration(self):
-        User.objects.create_user('testuser', 'test@example.com', 'testPass')
         self.client.login(username='testuser', password='testPass')
-
         # If the user if already logged in, redirect to index page...
         # Don't let him register again
         response = self.client.get(reverse('user_registration'))
         self.assertRedirects(response, reverse('root'))
-        self.client.logout()
 
         # Access the user registration page after logging out and try to register now
+        self.client.logout()
         response = self.client.get(reverse('user_registration'))
         self.assertEqual(response.status_code, 200)
 
         # @TODO: Try to register a user and verify its working
 
 
-from hyperkitty.views.accounts import last_views
-from hyperkitty.views.thread import thread_index
-from hyperkitty.views.list import archives
 
 class LastViewsTestCase(TestCase):
 
     def setUp(self):
         self.user = User.objects.create_user('testuser', 'test@example.com', 'testPass')
         self.client.login(username='testuser', password='testPass')
-        store = kittystore.get_store("sqlite:", debug=False)
+        store = kittystore.get_store(SettingsModule(), debug=False, auto_create=True)
+        self.client.defaults = {"kittystore.store": store,
+                                "HTTP_USER_AGENT": "testbot",
+                                }
+        # Create test data
         ml = FakeList("list@example.com")
         ml.subject_prefix = u"[example] "
         # Create 3 threads
@@ -113,14 +121,9 @@ class LastViewsTestCase(TestCase):
         msg4["In-Reply-To"] = "<id3>"
         msg4.set_payload("Dummy message")
         store.add_to_list(ml, msg4)
-        # Factory
-        defaults = {"kittystore.store": store, "HTTP_USER_AGENT": "testbot"}
-        self.factory = RequestFactory(**defaults)
 
     def test_profile(self):
-        request = self.factory.get(reverse('user_last_views'))
-        request.user = self.user
-        response = last_views(request)
+        response = self.client.get(reverse('user_last_views'))
         self.assertContains(response, "<td>dummy@example.com</td>",
                             count=2, status_code=200, html=True)
 
@@ -128,10 +131,8 @@ class LastViewsTestCase(TestCase):
         responses = []
         for msgnum in range(3):
             threadid = get_message_id_hash("<id%d>" % (msgnum+1))
-            request = self.factory.get(reverse('thread',
-                        args=["list@example.com", threadid]))
-            request.user = self.user
-            response = thread_index(request, "list@example.com", threadid)
+            response = self.client.get(reverse('thread', args=(
+                        "list@example.com", threadid)))
             responses.append(response)
         # There's always one icon in the right column, so all counts are +1
         self.assertContains(responses[0], "icon-eye-close", count=2, status_code=200)
@@ -140,132 +141,139 @@ class LastViewsTestCase(TestCase):
 
     def test_thread_list(self):
         now = datetime.datetime.now()
-        request = self.factory.get(reverse('archives_with_month', args=["list@example.com", now.year, now.month]))
-        request.user = self.user
-        response = archives(request, "list@example.com", now.year, now.month)
+        response = self.client.get(reverse('archives_with_month', args=(
+                    "list@example.com", now.year, now.month)))
         self.assertContains(response, "icon-eye-close",
                             count=2, status_code=200)
 
     def test_overview(self):
-        now = datetime.datetime.now()
-        request = self.factory.get(reverse('list_overview', args=["list@example.com"]))
-        request.user = self.user
-        response = archives(request, "list@example.com", now.year, now.month)
+        response = self.client.get(reverse('list_overview', args=["list@example.com"]))
         self.assertContains(response, "icon-eye-close",
-                            count=2, status_code=200)
+                            count=4, status_code=200)
 
 
-from hyperkitty.views.message import vote
 
 class MessageViewsTestCase(TestCase):
 
     def setUp(self):
         self.user = User.objects.create_user(
                 'testuser', 'test@example.com', 'testPass')
-        # Fake KittStore
-        class FakeMessage(object):
-            def __init__(self, h):
-                self.message_id_hash = h
-                self.list_name = "list@example.com"
-        self.store = Mock()
-        self.store.get_message_by_hash_from_list.side_effect = \
-                lambda l, h: FakeMessage(h)
-        defaults = {"kittystore.store": self.store}
-        self.factory = RequestFactory(**defaults)
+        self.client.login(username='testuser', password='testPass')
+        # use a temp variable below because self.client.session is actually a
+        # property which returns a new instance en each call :-/
+        session = self.client.session
+        session["user_id"] = u"testuser"
+        session.save()
+        self.store = kittystore.get_store(SettingsModule(),
+                                          debug=False, auto_create=True)
+        self.client.defaults = {"kittystore.store": self.store,
+                                "HTTP_USER_AGENT": "testbot",
+                                }
+        # Create a dummy message to test on
+        ml = FakeList("list@example.com")
+        msg = Message()
+        msg["From"] = "dummy@example.com"
+        msg["Message-ID"] = "<msg>"
+        msg.set_payload("Dummy message")
+        self.store.add_to_list(ml, msg)
 
 
     def test_vote_up(self):
-        request = self.factory.post("/vote", {"vote": "1"})
-        request.user = self.user
-        resp = vote(request, 'list@example.com', '123')
+        url = reverse('message_vote', args=("list@example.com",
+                      get_message_id_hash("msg")))
+        resp = self.client.post(url, {"vote": "1"})
         self.assertEqual(resp.status_code, 200)
-        v = Rating.objects.get(user=self.user, messageid="123",
-                               list_address='list@example.com')
-        self.assertEqual(v.vote, 1)
         result = json.loads(resp.content)
         self.assertEqual(result["like"], 1)
         self.assertEqual(result["dislike"], 0)
 
 
     def test_vote_down(self):
-        request = self.factory.post("/vote", {"vote": "-1"})
-        request.user = self.user
-        resp = vote(request, 'list@example.com', '123')
+        url = reverse('message_vote', args=("list@example.com",
+                      get_message_id_hash("msg")))
+        resp = self.client.post(url, {"vote": "-1"})
         self.assertEqual(resp.status_code, 200)
-        v = Rating.objects.get(user=self.user, messageid="123",
-                               list_address='list@example.com')
-        self.assertEqual(v.vote, -1)
         result = json.loads(resp.content)
         self.assertEqual(result["like"], 0)
         self.assertEqual(result["dislike"], 1)
 
 
     def test_vote_cancel(self):
-        v = Rating(list_address="list@example.com", messageid="m1", vote=1)
-        v.user = self.user
-        v.save()
-        v = Rating(list_address="list@example.com", messageid="m2", vote=-1)
-        v.user = self.user
-        v.save()
-        for msg in ["m1", "m2"]:
-            request = self.factory.post("/vote", {"vote": "0"})
-            request.user = self.user
-            resp = vote(request, 'list@example.com', msg)
+        ml = FakeList("list@example.com")
+        msg = Message()
+        msg["From"] = "dummy@example.com"
+        msg["Message-ID"] = "<msg1>"
+        msg.set_payload("Dummy message")
+        self.store.add_to_list(ml, msg)
+        msg.replace_header("Message-ID", "<msg2>")
+        self.store.add_to_list(ml, msg)
+        msg1 = self.store.get_message_by_id_from_list("list@example.com", "msg1")
+        msg1.vote(1, u"testuser")
+        msg2 = self.store.get_message_by_id_from_list("list@example.com", "msg2")
+        msg2.vote(-1, u"testuser")
+        self.assertEqual(msg1.likes, 1)
+        self.assertEqual(msg2.dislikes, 1)
+        for msg in (msg1, msg2):
+            url = reverse('message_vote', args=("list@example.com",
+                          msg.message_id_hash))
+            resp = self.client.post(url, {"vote": "0"})
             self.assertEqual(resp.status_code, 200)
-            try:
-                Rating.objects.get(user=self.user, messageid=msg,
-                                   list_address='list@example.com')
-            except Rating.DoesNotExist:
-                pass
-            else:
-                self.fail("Vote for msg %s should have been deleted" % msg)
+            self.assertEqual(msg.likes, 0)
+            self.assertEqual(msg.dislikes, 0)
             result = json.loads(resp.content)
             self.assertEqual(result["like"], 0)
             self.assertEqual(result["dislike"], 0)
 
 
     def test_unauth_vote(self):
-        request = self.factory.post("/vote", {"vote": "1"})
-        request.user = AnonymousUser()
-        resp = vote(request, 'list@example.com', '123')
+        self.client.logout()
+        url = reverse('message_vote', args=("list@example.com",
+                      get_message_id_hash("msg")))
+        resp = self.client.post(url, {"vote": "1"})
         self.assertEqual(resp.status_code, 403)
 
 
 
-from hyperkitty.views.list import archives
-
 class ListArchivesTestCase(TestCase):
 
     def setUp(self):
-#        self.store = Mock()
-#        self.store.get_threads.return_value = []
-#        self.store.get_list.side_effect = lambda n: FakeList(n)
-#        defaults = {"kittystore.store": self.store}
-#        self.factory = RequestFactory(**defaults)
-        self.factory = RequestFactory()
+        store = kittystore.get_store(SettingsModule(),
+                                     debug=False, auto_create=True)
+        self.client.defaults = {"kittystore.store": store,
+                                "HTTP_USER_AGENT": "testbot",
+                                }
+        # Create the list by adding a dummy message
+        ml = FakeList("list@example.com")
+        msg = Message()
+        msg["From"] = "dummy@example.com"
+        msg["Message-ID"] = "<msg>"
+        msg.set_payload("Dummy message")
+        store.add_to_list(ml, msg)
 
     def test_no_date(self):
         today = datetime.date.today()
-        request = self.factory.get("/archives")
-        response = archives(request, 'list@example.com')
+        response = self.client.get(reverse(
+                'archives_latest', args=['list@example.com']))
         final_url = reverse('archives_with_month',
                 kwargs={'mlist_fqdn': 'list@example.com',
                         'year': today.year,
                         'month': today.month,
                 })
-        self.assertEqual(response["location"], final_url)
+        self.assertRedirects(response, final_url)
 
 
 
-from hyperkitty.views.thread import reattach, reattach_suggest
-
 class ReattachTestCase(TestCase):
 
     def setUp(self):
         self.user = User.objects.create_user('testuser', 'test@example.com', 'testPass')
         self.user.is_staff = True
+        self.user.save()
         self.client.login(username='testuser', password='testPass')
-        self.store = kittystore.get_store("sqlite:", debug=False)
+        self.store = kittystore.get_store(SettingsModule(), debug=False, auto_create=True)
+        self.client.defaults = {"kittystore.store": self.store,
+                                "HTTP_USER_AGENT": "testbot",
+                                }
         ml = FakeList("list@example.com")
         ml.subject_prefix = u"[example] "
         # Create 2 threads
@@ -278,17 +286,14 @@ class ReattachTestCase(TestCase):
             msg.set_payload("Dummy message")
             msg["Message-ID-Hash"] = self.store.add_to_list(ml, msg)
             self.messages.append(msg)
-        # Factory
-        defaults = {"kittystore.store": self.store, "HTTP_USER_AGENT": "testbot"}
-        self.factory = RequestFactory(**defaults)
 
     def test_suggestions(self):
         threadid = self.messages[0]["Message-ID-Hash"]
-        request = self.factory.get(reverse('thread_reattach_suggest',
-                                   args=["list@example.com", threadid]))
         msg2 = self.store.get_message_by_id_from_list("list@example.com", "id2")
         self.store.search = Mock(return_value={"results": [msg2]})
-        response = reattach_suggest(request, "list@example.com", threadid)
+        self.store.search_index = True
+        response = self.client.get(reverse('thread_reattach_suggest',
+                                   args=["list@example.com", threadid]))
         self.store.search.assert_called_with(
             u'dummy message', 'list@example.com', 1, 50)
         other_threadid = self.messages[1]["Message-ID-Hash"]
@@ -298,72 +303,62 @@ class ReattachTestCase(TestCase):
     def test_reattach(self):
         threadid1 = self.messages[0]["Message-ID-Hash"]
         threadid2 = self.messages[1]["Message-ID-Hash"]
-        request = self.factory.post(reverse('thread_reattach',
-                                    args=["list@example.com", threadid2]),
-                                    data={"parent": threadid1})
-        request.user = self.user
-        response = reattach(request, "list@example.com", threadid2)
+        response = self.client.post(reverse('thread_reattach',
+                                   args=["list@example.com", threadid2]),
+                                   data={"parent": threadid1})
         now = datetime.datetime.now()
-        threads = self.store.get_threads("list@example.com",
+        threads = list(self.store.get_threads("list@example.com",
                 now - datetime.timedelta(days=1),
-                now + datetime.timedelta(days=1))
+                now + datetime.timedelta(days=1)))
         self.assertEqual(len(threads), 1)
         self.assertEqual(threads[0].thread_id, threadid1)
         expected_url = reverse('thread', args=["list@example.com", threadid1]) + "?msg=attached-ok"
-        self.assertEqual(response["location"], expected_url)
+        self.assertRedirects(response, expected_url)
 
     def test_reattach_manual(self):
         threadid1 = self.messages[0]["Message-ID-Hash"]
         threadid2 = self.messages[1]["Message-ID-Hash"]
-        request = self.factory.post(reverse('thread_reattach',
+        response = self.client.post(reverse('thread_reattach',
                                     args=["list@example.com", threadid2]),
                                     data={"parent": "",
                                           "parent-manual": threadid1})
-        request.user = self.user
-        response = reattach(request, "list@example.com", threadid2)
         now = datetime.datetime.now()
-        threads = self.store.get_threads("list@example.com",
+        threads = list(self.store.get_threads("list@example.com",
                 now - datetime.timedelta(days=1),
-                now + datetime.timedelta(days=1))
+                now + datetime.timedelta(days=1)))
         self.assertEqual(len(threads), 1)
         self.assertEqual(threads[0].thread_id, threadid1)
         expected_url = reverse('thread', args=["list@example.com", threadid1]) + "?msg=attached-ok"
-        self.assertEqual(response["location"], expected_url)
+        self.assertRedirects(response, expected_url)
 
     def test_reattach_invalid(self):
         threadid = self.messages[0]["Message-ID-Hash"]
-        request = self.factory.post(reverse('thread_reattach',
+        self.store.attach_to_thread = Mock()
+        response = self.client.post(reverse('thread_reattach',
                                     args=["list@example.com", threadid]),
                                     data={"parent": "invalid-data"})
-        request.user = self.user
-        self.store.attach_to_thread = Mock()
-        response = reattach(request, "list@example.com", threadid)
         self.assertFalse(self.store.attach_to_thread.called)
         now = datetime.datetime.now()
-        threads = self.store.get_threads("list@example.com",
+        threads = list(self.store.get_threads("list@example.com",
                 now - datetime.timedelta(days=1),
-                now + datetime.timedelta(days=1))
+                now + datetime.timedelta(days=1)))
         self.assertEqual(len(threads), 2)
-        errormsg = '<div class="flashmsgs"><div class="flashmsg-wrapper"><div class="alert alert-error">'
         self.assertContains(response, '<div class="alert alert-warning">',
                 count=1, status_code=200)
         self.assertContains(response, "Invalid thread id, it should look")
 
     def test_reattach_on_itself(self):
         threadid = self.messages[0]["Message-ID-Hash"]
-        request = self.factory.post(reverse('thread_reattach',
+        self.store.attach_to_thread = Mock()
+        response = self.client.post(reverse('thread_reattach',
                                     args=["list@example.com", threadid]),
                                     data={"parent": threadid})
-        request.user = self.user
-        self.store.attach_to_thread = Mock()
-        response = reattach(request, "list@example.com", threadid)
         self.assertFalse(self.store.attach_to_thread.called)
         now = datetime.datetime.now()
-        threads = self.store.get_threads("list@example.com",
+        threads = list(self.store.get_threads("list@example.com",
                 now - datetime.timedelta(days=1),
-                now + datetime.timedelta(days=1))
+                now + datetime.timedelta(days=1)))
         self.assertEqual(len(threads), 2)
-        errormsg = '<div class="flashmsgs"><div class="flashmsg-wrapper"><div class="alert alert-error">'
         self.assertContains(response, '<div class="alert alert-warning">',
                 count=1, status_code=200)
         self.assertContains(response, "Can&#39;t re-attach a thread to itself")
@@ -371,15 +366,11 @@ class ReattachTestCase(TestCase):
     def test_reattach_on_unknown(self):
         threadid = self.messages[0]["Message-ID-Hash"]
         threadid_unknown = "L36TVP2EFFDSXGVNQJCY44W5AB2YMJ65"
-        request = self.factory.post(reverse('thread_reattach',
+        self.store.attach_to_thread = Mock()
+        response = self.client.post(reverse('thread_reattach',
                                     args=["list@example.com", threadid]),
                                     data={"parent": threadid_unknown})
-        request.user = self.user
-        self.store.attach_to_thread = Mock()
-        response = reattach(request, "list@example.com", threadid)
         self.assertFalse(self.store.attach_to_thread.called)
-        now = datetime.datetime.now()
-        errormsg = '<div class="flashmsgs"><div class="flashmsg-wrapper"><div class="alert alert-error">'
         self.assertContains(response, '<div class="alert alert-warning">',
                 count=1, status_code=200)
         self.assertContains(response, "Unknown thread")
@@ -387,20 +378,79 @@ class ReattachTestCase(TestCase):
     def test_reattach_old_to_new(self):
         threadid1 = self.messages[0]["Message-ID-Hash"]
         threadid2 = self.messages[1]["Message-ID-Hash"]
-        request = self.factory.post(reverse('thread_reattach',
+        self.store.attach_to_thread = Mock()
+        response = self.client.post(reverse('thread_reattach',
                                     args=["list@example.com", threadid1]),
                                     data={"parent": threadid2})
-        request.user = self.user
-        self.store.attach_to_thread = Mock()
-        response = reattach(request, "list@example.com", threadid1)
         self.assertFalse(self.store.attach_to_thread.called)
         now = datetime.datetime.now()
-        threads = self.store.get_threads("list@example.com",
+        threads = list(self.store.get_threads("list@example.com",
                 now - datetime.timedelta(days=1),
-                now + datetime.timedelta(days=1))
+                now + datetime.timedelta(days=1)))
         self.assertEqual(len(threads), 2)
-        errormsg = '<div class="flashmsgs"><div class="flashmsg-wrapper"><div class="alert alert-error">'
         self.assertContains(response, '<div class="alert alert-error">',
                 count=1, status_code=200)
         self.assertContains(response, "Can&#39;t attach an older thread to a newer thread.",
                 count=1, status_code=200)
+
+
+
+class PrivateArchivesTestCase(TestCase):
+
+    def setUp(self):
+        self.tmpdir = mkdtemp(prefix="hyperkitty-testing-")
+        self.user = User.objects.create_user('testuser', 'test@example.com', 'testPass')
+        settings = SettingsModule()
+        settings.KITTYSTORE_SEARCH_INDEX = self.tmpdir
+        self.store = kittystore.get_store(settings, debug=False, auto_create=True)
+        self.client.defaults = {"kittystore.store": self.store,
+                                "HTTP_USER_AGENT": "testbot",
+                                }
+        ml = FakeList("list@example.com")
+        ml.subject_prefix = u"[example] "
+        ml.archive_policy = ArchivePolicy.private
+        msg = Message()
+        msg["From"] = "dummy@example.com"
+        msg["Message-ID"] = "<msgid>"
+        msg["Subject"] = "Dummy message"
+        msg.set_payload("Dummy message")
+        msg["Message-ID-Hash"] = self.msgid = self.store.add_to_list(ml, msg)
+
+    def tearDown(self):
+        rmtree(self.tmpdir)
+
+
+    def _do_test(self, url, query={}):
+        response = self.client.get(url, query)
+        self.assertEquals(response.status_code, 403)
+        self.client.login(username='testuser', password='testPass')
+        # use a temp variable below because self.client.session is actually a
+        # property which returns a new instance en each call :-/
+        session = self.client.session
+        session["subscribed"] = ["list@example.com"]
+        session.save()
+        response = self.client.get(url, query)
+        self.assertContains(response, "Dummy message", status_code=200)
+
+
+    def test_month_view(self):
+        now = datetime.datetime.now()
+        self._do_test(reverse('archives_with_month', args=["list@example.com", now.year, now.month]))
+
+    def test_overview(self):
+        self._do_test(reverse('list_overview', args=["list@example.com"]))
+
+    def test_thread_view(self):
+        self._do_test(reverse('thread', args=["list@example.com", self.msgid]))
+
+    def test_message_view(self):
+        self._do_test(reverse('message_index', args=["list@example.com", self.msgid]))
+
+    def test_search_list(self):
+        self._do_test(reverse('search'), {"list": "list@example.com", "query": "dummy"})
+
+    def test_search_all_lists(self):
+        # When searching all lists, we only search public lists regardless of
+        # the user's subscriptions
+        response = self.client.get(reverse('search'), {"query": "dummy"})
+        self.assertNotContains(response, "Dummy message", status_code=200)
