@@ -24,7 +24,7 @@ from django import forms
 from django.core import validators
 from django.contrib.auth.models import User
 from django.utils.safestring import mark_safe
-from django.utils.translation import ugettext, ugettext_lazy
+from django.utils.translation import ugettext_lazy
 
 from hyperkitty.models import UserProfile
 
@@ -109,8 +109,8 @@ class AttachmentFileInput(forms.FileInput):
     %(input)s <a href="#" title="%(rm_text)s">(-)</a>
 </span>
 <span class="attach-files"></span>
-<a class="attach-files-first">%(attach_first_text)s</a>
-<a class="attach-files-add">%(attach_another_text)s</a>
+<a href="#" class="attach-files-first">%(attach_first_text)s</a>
+<a href="#" class="attach-files-add">%(attach_another_text)s</a>
 """
 
     def render(self, name, value, attrs=None):
